package com.circuitengine.model.Components;

import java.awt.Graphics;

import com.circuitengine.view.CircuitEngineApplet;
import com.circuitengine.view.CircuitEnginePanel;

/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
public final class Inductor extends ElectricComponent {
  private CircuitEnginePanel parentCircuitEnginePanel;
  public Inductor(int x, int y,CircuitEnginePanel p){
    super(x,y);
    this.parentCircuitEnginePanel=p;
    init();
  }
  public void init(){
    secondary="Current";
    primary="Inductance";
    primaryUnit="Henry";
    primaryUnitPlural="Henries";
    primarySymbol="L";
    primaryUnitSymbol="H";
    secondaryUnitSymbol="A";
    preselectedPrimaryPrefix="mH";
  }
  public void draw(Graphics g) {
    super.draw(g);
    int x1=head.getX();
    int x2=tail.getX();
    int y1=head.getY();
    int y2=tail.getY();
    int rise = y2 - y1;
    int run = x2 - x1;
        int distanceSquared = (rise * rise + run * run);
        double distance=Math.sqrt(distanceSquared);
        double divisor;
        double seperationRise,seperationRun;
       
        divisor = Math.sqrt(distanceSquared) / 35.0;
        if (distanceSquared < 35 * 35) {
            divisor = 1.0;
        }
        double middlex1 = x1 + run / 2.0 - run / 2.0 / divisor;
        double middley1 = y1 + rise / 2.0 - rise / 2.0 / divisor;
        double middlex2 = x2 - run / 2.0 + run / 2.0 / divisor;
        double middley2 = y2 - rise / 2.0 + rise / 2.0 / divisor;
        seperationRise = middley2 - middley1;
        seperationRun = middlex2 - middlex1;
        double seperationDistance=Math.sqrt(sqr(seperationRun)+sqr(seperationRise));
        g.drawLine(x1, y1, (int)Math.round(middlex1), (int)Math.round(middley1));
        g.drawLine((int)Math.round(middlex2), (int)Math.round(middley2), x2, y2);
        for(int i=0;i<4;i++){
        g.drawArc((int)Math.round((2*i+1)*(middlex2-middlex1)/8.0+middlex1-seperationDistance/8.0),
                  (int)Math.round((2*i+1)*(middley2-middley1)/8.0+middley1-seperationDistance/8.0),
                  (int)Math.round(seperationDistance/4.0),
                   (int)Math.round(seperationDistance/4.0),
                  (int)Math.round(-Math.atan2(rise,run)/Math.PI*180),
                  (int)(180));
        }
       /* g.drawLine((int) (middlex1 - rise / divisor), (int) (middley1 + run / divisor),
                   (int) (middlex1 + rise / divisor),
                   (int) (middley1 - run / divisor));
        g.drawLine((int) (middlex2 - rise / divisor), (int) (middley2 + run / divisor),
                   (int) (middlex2 + rise / divisor),
                   (int) (middley2 - run / divisor));*/
        int a = 9;
        if (parentCircuitEnginePanel.drawPlus){
           g.drawString("+",(int)(middlex1-a*run/distance-a*rise/distance),
                        (int)(middley1-a*rise/distance+a*run/distance));
        }
        
    
  }
}
