package com.circuitengine.model.Components;

import java.awt.Graphics;

import com.circuitengine.model.TiePoint;

/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
public final class Resistor extends ElectricComponent{
  double resistance;
  public static String ohm="\u2126";
  public Resistor(int x, int y){
    super(x,y);
    init();
  }
  public void init(){
    primary="Resistance";
    primaryUnit="Ohm";
    primaryUnitPlural="Ohms";
    primarySymbol="R";
    primaryUnitSymbol=ohm;
    preselectedPrimaryPrefix=ohm;
  }
  public void draw(Graphics g) {
    super.draw(g);
    int x1=head.getX();
    int x2=tail.getX();
    int y1=head.getY();
    int y2=tail.getY();
    int rise = y2 - y1;
    int run = x2 - x1;
    int distanceSquared = (rise * rise + run * run);
    double divisor;
    double seperationRise, seperationRun;
    divisor = Math.sqrt(distanceSquared) / 35.0;
    if (distanceSquared < 35 * 35) {
        divisor = 1.0;
    }
    double middlex1 = x1 + run / 2.0 - run / 2.0 / divisor;
    double middley1 = y1 + rise / 2.0 - rise / 2.0 / divisor;
    double middlex2 = x2 - run / 2.0 + run / 2.0 / divisor;
    double middley2 = y2 - rise / 2.0 + rise / 2.0 / divisor;
    seperationRise = middley2 - middley1;
    seperationRun = middlex2 - middlex1;
    g.drawLine(x1, y1, (int) middlex1, (int) middley1);
    g.drawLine((int) middlex2, (int) middley2, x2, y2);
    g.drawLine((int) middlex1, (int) middley1, (int) (middlex1 + seperationRun / 12.0 - seperationRise / 4.0),
               (int) (middley1 + seperationRise / 12.0 + seperationRun / 4.0));
    g.drawLine((int) (middlex2 - seperationRun / 12.0 + seperationRise / 4.0),
               (int) (middley2 - seperationRise / 12.0 - seperationRun / 4.0), (int) middlex2, (int) middley2);
    for (int i = 0;i < 2;i++) {
        g.drawLine((int) (middlex1 + 3 * seperationRun / 12.0 + seperationRise / 4.0 + 4 * i * seperationRun / 12.0),
                   (int) (middley1 + 3 * seperationRise / 12.0 - seperationRun / 4.0 + 4 * i * seperationRise / 12.0),
                   (int) (middlex1 + 5 * seperationRun / 12.0 - seperationRise / 4.0 + 4 * i * seperationRun / 12.0),
                   (int) (middley1 + 5 * seperationRise / 12.0 + seperationRun / 4.0 + 4 * i * seperationRise / 12.0));
    }
    for (int i = 0;i < 3;i++) {
        g.drawLine((int) (middlex1 + seperationRun / 12.0 - seperationRise / 4.0 + 4 * i * seperationRun / 12.0),
                   (int) (middley1 + seperationRise / 12.0 + seperationRun / 4.0 + 4 * i * seperationRise / 12.0),
                   (int) (middlex1 + 3 * seperationRun / 12.0 + seperationRise / 4.0 + 4 * i * seperationRun / 12.0),
                   (int) (middley1 + 3 * seperationRise / 12.0 - seperationRun / 4.0 + 4 * i * seperationRise / 12.0));
    }
  }
}
