package com.circuitengine.view;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.LinkedList;

import javax.swing.JPanel;
import javax.swing.Timer;

import com.circuitengine.model.Circuit;
import com.circuitengine.model.TiePoint;
import com.circuitengine.model.Components.ACCurrentSource;
import com.circuitengine.model.Components.ACVoltageSource;
import com.circuitengine.model.Components.Ammeter;
import com.circuitengine.model.Components.Battery;
import com.circuitengine.model.Components.CapacitanceMeter;
import com.circuitengine.model.Components.Capacitor;
import com.circuitengine.model.Components.CurrentIntegrator;
import com.circuitengine.model.Components.CurrentSource;
import com.circuitengine.model.Components.ElectricComponent;
import com.circuitengine.model.Components.InductanceMeter;
import com.circuitengine.model.Components.Inductor;
import com.circuitengine.model.Components.Ohmmeter;
import com.circuitengine.model.Components.Resistor;
import com.circuitengine.model.Components.Switch;
import com.circuitengine.model.Components.VoltageIntegrator;
import com.circuitengine.model.Components.Voltmeter;
import com.circuitengine.model.Components.Wire;

/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
public final class CircuitEnginePanel extends JPanel {
  boolean myIsMovingPoint;
  boolean myIsMovingComponent;
  boolean myIsDrawing;
  int myMouseX,myMouseY;
  boolean myButtonState;//true if pressed
  boolean myMouseIsEntered;//true if mouse is in DrawingPanel
  public final static int x1 = 4;//30;
  public final static int y1 = 4;//30;
  public final static double deltax = 30;
  public final static double deltay = 30;
  public static int numx = (int)((800 - x1) / deltax);
  public static int numy = (int)((600 - y1) / deltay);
  HintPanel myHintPanel;
  public ElectricComponent myTempComponent;
  public ElectricComponent myNearestComponent;
  public ElectricComponent myDraggedComponent;
  public Switch myNearestSwitch;
  public TiePoint myNearestTiePoint;
  public TiePoint myDraggedTiePoint;
  String mySelectedButton;
  public Circuit myCircuit;
  public LinkedList<PhasePlane> phasePlanes;
  public HashMap<Point,TiePoint> myGridPoints;
  public final DrawingPanel dp;
  public boolean isBusy;
  public boolean waiting;
  public boolean isPlaying;
  public String defaultCapacitorErrorString="1.0E-9";
  public String defaultInductorErrorString="1.0E-9";
  public String capacitorErrorString=defaultCapacitorErrorString;
  public String inductorErrorString=defaultInductorErrorString;
  public double defaultCapacitorError=Math.pow(10, -9);
  public double defaultInductanceError=Math.pow(10, -9);//was 10^-11
  public double capacitorError=defaultCapacitorError;
  public double inductanceError=defaultInductanceError;
  public boolean drawPlus=true;
  public boolean timeOption=true;
  public CircuitEngineApplet myParentCircuitEngineApplet;
  public CircuitEnginePanel(CircuitEngineApplet parentCircuitEngineApplet) {
    super();
    myParentCircuitEngineApplet=parentCircuitEngineApplet;
    myCircuit = new Circuit();
    phasePlanes=new LinkedList<PhasePlane>();
    myGridPoints = new HashMap<Point,TiePoint>();
    this.setLayout(new BorderLayout());
    //this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    myHintPanel = new HintPanel(this);
    //JPanel bp = new ButtonPanel(this);
    //bp.setLayout(new BorderLayout());
    this.add(new ButtonPanel(this),BorderLayout.NORTH);
    //JPanel outerdp=new JPanel();
   //outerdp.setLayout(new BorderLayout());
    //this.add(bp);
    dp= new DrawingPanel(this);
    Timer my_timer = new Timer(50, new ActionListener() {
        public void actionPerformed(ActionEvent e) {
              if (isBusy||!isPlaying){
                return;
              }
              
               // System.out.println("a");
              synchronized(dp){//##changed from synchronized this on Dec 29 2008
              dp.updateCircuit();
              dp.repaint();
              }
              //System.out.println("b");
            }});
    
    
    
    my_timer.setRepeats(true);
    my_timer.start();
    this.add(dp,BorderLayout.CENTER);
    myHintPanel.startingHint();
    //JPanel southPanel= new JPanel();
    //outerdp.add(dp,BorderLayout.CENTER);
    //this.add(outerdp);
    //southPanel.add(myHintPanel,BorderLayout.SOUTH);
    //this.add(myHintPanel);
    this.add(myHintPanel,BorderLayout.SOUTH);
    //this.add(southPanel,BorderLayout.SOUTH);
    this.revalidate();
    setIsMovingPoint(false);
    setIsMovingComponent(false);
    setIsDrawing(false);
  }
  public void setIsMovingPoint(final boolean theIsMovingPoint) {
    this.myIsMovingPoint = theIsMovingPoint;
  }
  public boolean getIsMovingPoint(){
    return myIsMovingPoint;
  }
  public boolean getIsDrawing(){
    return myIsDrawing;
  }
  public void setIsDrawing(final boolean theIsDrawing) {
    this.myIsDrawing = theIsDrawing;
  }
  public boolean getIsMovingComponent(){
    return myIsMovingComponent;
  }
  public void setIsMovingComponent(final boolean theIsMovingComponent){
    this.myIsMovingComponent=theIsMovingComponent;
  }
  public void setMouseX(final int theMouseX) {
    myMouseX = theMouseX;
  }
  public int getMouseX(){
    return myMouseX;
  }
  public void setMouseY(final int theMouseY) {
    myMouseY = theMouseY;
  }
  public int getMouseY(){
    return myMouseY;
  }
  public void setButtonState(final boolean theButtonState){
    myButtonState = theButtonState;
  }
  public void setMouseEntered(final boolean theMouseEntered) {
    myMouseIsEntered = theMouseEntered;
  }
  public boolean getMouseEntered(){
    return myMouseIsEntered;
  }
  public void setHintDrawing(){
    if (!myHintPanel.hint.equals("drawing")){
      myHintPanel.drawingHint();
    }
  }
  public void setHintStarting(){
    if (!myHintPanel.hint.equals("starting")){
      myHintPanel.startingHint();
    }
  }
  public void hintNearTiePoint(){
    if (!myHintPanel.hint.equals("tiePoint")){
      myHintPanel.nearTiePointHint();
    }
  }
  public void hintNearComponent(){
    if (!myHintPanel.hint.equals("component")){
      myHintPanel.nearComponentHint();
    }
  }
  public void hintNearSwitch(){
    if (!myHintPanel.hint.equals("switch")){
      myHintPanel.nearSwitchHint();
    }
  }
  public void newTempComponent(){
    //try {
     boolean demo=true;
      String s =mySelectedButton.replaceAll(" ", "");
      if (s.equals("Ammeter")){
        myTempComponent=new Ammeter(nearestX(),nearestY());
      }else if (s.equals("Battery")){
        myTempComponent=new Battery(nearestX(),nearestY());
        if (demo){
         myTempComponent.primaryValue=4.56;//3.0;//4.56;
         myTempComponent.enteredPrimaryValue="4.56";
        }
      }else if (s.equals("CapacitanceMeter")){
        myTempComponent=new CapacitanceMeter(nearestX(),nearestY());
      }else if (s.equals("Capacitor")){
        myTempComponent=new Capacitor(nearestX(),nearestY(),this);
        if (demo){
          myTempComponent.primaryValue=0.00000123;
          myTempComponent.enteredPrimaryValue="1.23";
         }
      }else if (s.equals("CurrentSource")){
        myTempComponent=new CurrentSource(nearestX(),nearestY());
        if (demo){
          myTempComponent.primaryValue=.0345;
          myTempComponent.enteredPrimaryValue="34.5";
         }
      }else if (s.equals("CurrentIntegrator")){
        myTempComponent=new CurrentIntegrator(nearestX(),nearestY());
      }else if (s.equals("VoltageIntegrator")){
        myTempComponent=new VoltageIntegrator(nearestX(),nearestY());
      }else if (s.equals("InductanceMeter")){
        myTempComponent=new InductanceMeter(nearestX(),nearestY());
      }else if (s.equals("Inductor")){
        myTempComponent=new Inductor(nearestX(),nearestY(),this);
        if (demo){
          myTempComponent.primaryValue=0.00234;
          myTempComponent.enteredPrimaryValue="2.34";
         }
      }else if (s.equals("Ohmmeter")){
        myTempComponent=new Ohmmeter(nearestX(),nearestY());
      }else if (s.equals("Resistor")){
        myTempComponent=new Resistor(nearestX(),nearestY());
        if (demo){
          myTempComponent.primaryValue=5.67;//5.0;//5.67;
          myTempComponent.enteredPrimaryValue="5.67";
         }
      }else if (s.equals("Switch")){
        myTempComponent=new Switch(nearestX(),nearestY());
      }else if (s.equals("Voltmeter")){
        myTempComponent=new Voltmeter(nearestX(),nearestY());
      }else if (s.equals("Wire")){
        myTempComponent=new Wire(nearestX(),nearestY());
      }else if (s.equals("ACCurrentSource")){
        myTempComponent=new ACCurrentSource(nearestX(),nearestY());
        if (demo){
          myTempComponent.primaryValue=0.0125;
          myTempComponent.enteredPrimaryValue="12.5";
          myTempComponent.frequency=60.0;
          myTempComponent.enteredFrequency="60";
          myTempComponent.phaseShift=0.0;
          myTempComponent.enteredPhaseShift="0";
         }
      }else if (s.equals("ACVoltageSource")){
        myTempComponent=new ACVoltageSource(nearestX(),nearestY());
        if (demo){
          myTempComponent.primaryValue=32.6;
          myTempComponent.enteredPrimaryValue="32.6";
          myTempComponent.frequency=60.0;
          myTempComponent.enteredFrequency="60";
          myTempComponent.phaseShift=0.0;
          myTempComponent.enteredPhaseShift="0";
         }
      }
      //Class cl = Class.forName("Model.Components."+mySelectedButton.replaceAll(" ", ""));
      //Constructor cons = cl.getConstructor(Integer.TYPE,Integer.TYPE);
      //Point p = new Point(nearestX(),nearestY());
      //myTempComponent=(ElectricComponent)cons.newInstance(nearestX(),nearestY());
      //myTempComponent=(ElectricComponent)cons.newInstance(nearestX(),nearestY());
    /*} catch (InvocationTargetException e) { 
      //System.out.println("InvocationTargetException");
    } catch (NoSuchMethodException e) {  
      //System.out.println("NoSuchMethodException");
    } catch (ClassNotFoundException e) {
      //System.out.println("ClassNotFoundException");
    } catch (IllegalAccessException e) {
      //System.out.println("IllegalAccessException");
    } catch (InstantiationException e) {
      //System.out.println("InstantiationException");
    }*/
  }
  public void setSelectedButton(final String theSelectedButton) {
    mySelectedButton=theSelectedButton;
  }
  public int nearestX(){
    long result = Math.round((getMouseX()-x1)/deltax);
    result=Math.max(result, 0);
    result = Math.min(result,numx-1);
    return (int)Math.round(x1+result*deltax);
  }
  public int nearestY(){
    long result= Math.round((getMouseY()-y1)/deltay);
    result=Math.max(result, 0);
    result = Math.min(result, numy-1);
    return (int)Math.round(y1+result*deltay);
  }
  public void addElectricComponent(){
    Point p1 = new Point(myTempComponent.head.getX(),myTempComponent.head.getY());
    if (!myGridPoints.containsKey(p1)){
      myGridPoints.put(p1, myTempComponent.head);
    }
    myTempComponent.setHead(myGridPoints.get(p1));
    Point p2 = new Point(nearestX(),nearestY());
    if (!myGridPoints.containsKey(p2)){
      myGridPoints.put(p2, myTempComponent.tail);
    }
    myTempComponent.setTail(myGridPoints.get(p2));
    //myTempComponent.connectHeadAndTail();
    myCircuit.electricComponents.add(myTempComponent);
  }
  public boolean nearTiePoint(){
    boolean result = false;
    if (myGridPoints.containsKey(new Point(nearestX(),nearestY()))
        &&sqr(getMouseX()-nearestX())+sqr(getMouseY()-nearestY())<10*10){
      result = true;
      myNearestTiePoint=myGridPoints.get(new Point(nearestX(),nearestY()));
    }
    return result;
  }
  public double sqr(double x){
    return x*x;
  }
  public boolean nearComponent(){
    int x3=getMouseX();
    int y3=getMouseY();
    double mindistance=Double.MAX_VALUE;
    boolean result = false;
    for (ElectricComponent electricComponent:myCircuit.electricComponents){
      int x1=electricComponent.head.getX();
      int y1=electricComponent.head.getY();
      int x2=electricComponent.tail.getX();
      int y2=electricComponent.tail.getY();
      double u=((x3-x1)*(x2-x1)+(y3-y1)*(y2-y1))/(sqr(x2-x1)+sqr(y2-y1));
      double x=x1+u*(x2-x1);
      double y = y1+u*(y2-y1);
      if (0<u && u<1 && sqr(x3-x)+sqr(y3-y)<300){
        if (sqr(x3-x)+sqr(y3-y)<mindistance){
          mindistance=sqr(x3-x)+sqr(y3-y);
          myNearestComponent=electricComponent;
          double d1=sqr(electricComponent.head.getX()-x3)+sqr(electricComponent.head.getY()-y3);
          double d2=sqr(electricComponent.tail.getX()-x3)+sqr(electricComponent.tail.getY()-y3);
          if (d1<d2){
            myNearestTiePoint = electricComponent.head;
          } else {
            myNearestTiePoint = electricComponent.tail;
          }
        }
        result= true;
      }
    }
    return result;
  }
  public boolean nearSwitch(){
    int x3=getMouseX();
    int y3=getMouseY();
    double mindistance=Double.MAX_VALUE;
    boolean result = false;
    for (ElectricComponent electricComponent:myCircuit.electricComponents){
      if (electricComponent.getClass()!=Switch.class){
        continue;
      }
      int x1=electricComponent.head.getX();
      int y1=electricComponent.head.getY();
      int x2=electricComponent.tail.getX();
      int y2=electricComponent.tail.getY();
      double u=((x3-x1)*(x2-x1)+(y3-y1)*(y2-y1))/(sqr(x2-x1)+sqr(y2-y1));
      double x=x1+u*(x2-x1);
      double y = y1+u*(y2-y1);
      if (0<u && u<1 && sqr(x3-0.5*(x1+x2))+sqr(y3-0.5*(y1+y2))<300){
        if (sqr(x3-x)+sqr(y3-y)<mindistance){
          mindistance=sqr(x3-x)+sqr(y3-y);
          myNearestSwitch=(Switch)electricComponent;
        }
        result= true;
      }
    }
    return result;
  }
}