package com.circuitengine.view;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.circuitengine.model.Components.Capacitor;
import com.circuitengine.model.Components.ElectricComponent;
import com.circuitengine.model.Components.Inductor;

/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
public final class ChangePrimaryValueFrame extends JFrame implements ActionListener{
  //private String ohm="\u2126";
  private String micro="\u03BC";
  private String[] siprefixes={"Y","Z","E","P","T","G","M","k","","m",micro,"n","p","f","a","z","y"};
  private JButton okayButton,cancelButton;
  private String okayButtonString ="Okay";
  private String cancelButtonString = "Cancel";
  private JPanel buttonPanel;
  private JPanel entryPanel;
  private JTextField primaryValueField;
  private JComboBox unitComboBox;
  private ElectricComponent component;
  private String[] prefixes;
  private HashMap prefixMap;
  private JLabel unitLabel;
  private CircuitEnginePanel myParent;
  public ChangePrimaryValueFrame(CircuitEnginePanel parent,int x,int y,ElectricComponent ec){
      super();
      component=ec;
      myParent=parent;
      //this.setSize(300,200);
      this.setLocationRelativeTo(null);
      this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
      this.setLayout(new BorderLayout());
      okayButton=new JButton(okayButtonString);
      cancelButton=new JButton(cancelButtonString);
      okayButton.addActionListener(this);
      cancelButton.addActionListener(this);
      buttonPanel=new JPanel();
      buttonPanel.setLayout(new FlowLayout());
      buttonPanel.add(okayButton);
      buttonPanel.add(cancelButton);
      entryPanel=new JPanel();
      entryPanel.setLayout(new FlowLayout());
      entryPanel.add(new JLabel(ec.primarySymbol+" = "));
      primaryValueField=new JTextField(15);
      if (ec.enteredPrimaryValue!=null){
        primaryValueField.setText(ec.enteredPrimaryValue);
      }
      entryPanel.add(primaryValueField);
      prefixes=new String[siprefixes.length];
      prefixMap=new HashMap();
      int power=24;
      for(int i=0;i<siprefixes.length;i++){
        prefixes[i]=siprefixes[i]+ec.primaryUnitSymbol;
        prefixMap.put(prefixes[i],power);
        //System.out.println(prefixes[i]);
        power-=3;
      }
      unitComboBox=new JComboBox(prefixes);
      if (ec.selectedPrimaryPrefix!=null){
        unitComboBox.setSelectedItem(ec.selectedPrimaryPrefix);
      }else{
        unitComboBox.setSelectedItem(ec.preselectedPrimaryPrefix);
      }
      unitComboBox.addActionListener(this);
      entryPanel.add(unitComboBox);
      unitLabel=new JLabel();//"("+electricComponent.primaryUnitPlural+")");
      String s=(String)unitComboBox.getSelectedItem();
      if (s.equals(ec.primaryUnitSymbol)){
          unitLabel.setText("("+ec.primaryUnitPlural+")");
        }else{
          unitLabel.setText("("+"10E"+(Integer)prefixMap.get(s)+" "+ec.primaryUnitPlural+")");
        }
      entryPanel.add(unitLabel);
      this.add(entryPanel,BorderLayout.CENTER);
      this.add(buttonPanel,BorderLayout.SOUTH);
      this.pack();
      this.setVisible(true);
  }
  public void actionPerformed(ActionEvent e) {
      String actionCommand = e.getActionCommand();
      if (actionCommand.equals("Cancel")) {
          dispose();
      } else if (actionCommand.equals("Okay")) {
          synchronized(myParent.dp){
            if (myParent.myCircuit.electricComponents.contains(component)){
                String text=primaryValueField.getText();
                if (text.equals("")){
                  component.primaryValue=null;
                  component.enteredPrimaryValue=text;
                  component.selectedPrimaryPrefix=(String)unitComboBox.getSelectedItem();
                  dispose();
                  myParent.repaint();
                }else {
                try{
                    String s=(String)unitComboBox.getSelectedItem();
                    if ((component.getClass()==Capacitor.class||component.getClass()==Inductor.class)
                        &&Double.parseDouble(text)==0.0){
                      JOptionPane.showMessageDialog(this, "Cannot be zero.");
                      return;
                    }
                    component.primaryValue=Double.parseDouble(text)*Math.pow(10,(Integer)prefixMap.get(s));
                    component.enteredPrimaryValue=text;
                    component.selectedPrimaryPrefix=(String)unitComboBox.getSelectedItem();
                    dispose();
                    myParent.repaint();
                }catch (NumberFormatException errorexcept){
                    //System.out.println("invalid number exception");
                    JOptionPane.showMessageDialog(this, "Not a valid number.");
                }
                }
             //Double.parseDouble(aString)System.out.println(5);
            }else{
                //System.out.println("no such component");
                JOptionPane.showMessageDialog(this, "That component no longer exists.");
                dispose();
            }
          }
      } else if(actionCommand.equals("comboBoxChanged")){
        //System.out.println(actionCommand);
        //System.out.println("changed");
        String s=(String)unitComboBox.getSelectedItem();
        //System.out.println(s);
        if (s.equals(component.primaryUnitSymbol)){
          unitLabel.setText("("+component.primaryUnitPlural+")");
        }else{
          unitLabel.setText("("+"10E"+(Integer)prefixMap.get(s)+" "+component.primaryUnitPlural+")");
        }
        pack();
      }else{
          //System.out.println("Unexpected Error in Configure Resistor Window");
      }
  }
}